#!/usr/bin/env python
# -*- coding: utf-8 -*-
#title           :menu.py
#description     :This program displays an interactive menu on CLI
#author          :
#date            :
#version         :0.1
#usage           :python menu.py
#notes           :
#python_version  :2.7.6  
#=======================================================================
 
# Import the modules needed to run the script.
import sys, os, re, sys, pickle
from ioboard import *#IOBoard
 
# Main definition - constants
main_menu_ctrl='main_menu'
menu_actions  = {}  
config_files_generated=False
 
# =======================
#     MENUS FUNCTIONS
# =======================
    
def clear_screen():
    """Clear the screen"""
    
    #Check which OS ar we running and clear the screen
    if (os.name=='nt'):
        os.system('CLS')
    else:
        os.system('clear')
    return
    
    
def print_error(str_error):
    """Print an error message and wait to continue
    :param str_error: String with error description
    """
    print ("Error! " + str_error)
    print ("Please try again.")
    raw_input(" Press ENTER to continue...")



def print_ports(board):
    """Print the information of the actual configuration of the board showing a tabulated table with
    the port number, the selected function, the list of available options and the description for the port
    """
    print_info=[["PORT NUMBER", "FUNCTION", "OPTIONS", "DESCRIPTION"]]
        
    for i in range(0, NUM_PORTS):
        print_info.append([board.IO_port_table[i].name,port_type_display_str[board.IO_port_table[i].actual_function],board.IO_port_table[i].get_str_port_options(),board.IO_port_table[i].description])
            
    col_width = max(len(word) for row in print_info for word in row) + 5  # padding
    string_line_simple = "-" * ((col_width+2)*4) 
    string_line_double = "=" * ((col_width+2)*4)  
        
    print "| ".join(word.ljust(col_width) for word in print_info[0]) + "|"
    print (string_line_double)
            
    for row in print_info[1:(NUM_PORTS+1)]:
        print "| ".join(word.ljust(col_width) for word in row) + "|"
            
    print (string_line_double)
        
def print_board_info(board):
    """Print the information of the board (name, description and board product code"""
    board._calculate_product_id()
    board._calculate_used_ports_id()
    print ("BOARD NAME: " + board.name)
    print ("BOARD DESCRIPTION: " + board.description)
    print ("BOARD PRODUCT CODE: " + str(board.used_ports) + "__" + str(board.product_code))
    print ("\n")

 
# Main menu
def main_menu():
    clear_screen()
    print_board_info(my_board)
    print_ports(my_board)
    print ("\n")
    print ("I/O Creator:\n")
    print ("Select option:")
    print ("1. Set Board Info")
    print ("2. Change I/O Type")
    print ("3. Generate config files")
    print ("0. Quit\n")
    choice = raw_input(" >>  ")
    try:
        menu_actions[choice]()
    except KeyError:
        print_error ("Invalid selection.")


#menu_board_info -- Ask for the board information and save the data checking for input errors        
def menu_board_info():
    global config_files_generated 
    print ("Define board name (max:15 character - Only letters, numbers, and underscore) ")
    board_name = raw_input(" >>  ")
    if len(board_name) > 15:
        print_error ("15 character max.")
    else:
        if not re.match("^[A-Za-z0-9_-]*$", board_name):
            print_error ("Only letters, numbers, and underscore allowed.")
        else:
            print ("Define aditional description (max:15 character - Only letters, numbers, and underscore) ")
            board_descr = raw_input(" >>  ")
            if len(board_descr) > 25:
                print_error ("25 character max.")
            else:
                if not re.match("^[A-Za-z0-9_-]*$", board_descr):
                    print_error ("Only letters, numbers, and underscore allowed.")
                else:
                    my_board.set_name(board_name)
                    my_board.set_description(board_descr)
                    config_files_generated=False
    

#menu_iochange -- Ask for a port information and save the data checking for input errors    
def menu_iochange():
    global config_files_generated 
    print ("Select I/O port ( 0-15 )")
    ioport_selected = raw_input(" >>  ")
    if not re.match("^[0-9]*$", ioport_selected):
        print_error ("Invalid port number.")
    else:
        ioport_selected=int(ioport_selected)
        if (not my_board.check_port_number(ioport_selected)):
            print_error ("Invalid port number.")
        else:
            print ("Select I/O type ")
            print (my_board.IO_port_table[ioport_selected].get_str_port_options())
            ioport_type = raw_input(" >>  ")
            
            if (not my_board.IO_port_table[ioport_selected].check_selected_option(ioport_type)):
                print_error ("Invalid port type.")
            else:
                if (ioport_type=="NU"):
                    my_board.IO_port_table[ioport_selected].set_actual_function(ioport_type)
                    my_board.IO_port_table[ioport_selected].set_description("")
                else:
                    print ("Define port name (max:15 character - Only letters, numbers, and underscore) ")
                    ioport_name = raw_input(" >>  ")
                    if len(ioport_name) > 15:
                        print_error ("15 character max.")
                    else:
                        if not re.match("^[A-Za-z0-9_-]*$", ioport_name):
                            print_error ("Only letters, numbers, and underscore allowed.")
                        else:
                            my_board.IO_port_table[ioport_selected].set_actual_function(ioport_type)
                            my_board.IO_port_table[ioport_selected].set_description(ioport_name)
                            config_files_generated=False
    return 'main_menu'

#menu_generate_files -- Generate the xml file for the Lagarto server and the header file for the arduino sketch
# and saves the board object information in a .pkl file
def menu_generate_files():
    global config_files_generated 
    if (not os.path.exists(my_board.get_filename_str())):
        os.makedirs(my_board.get_filename_str())
    my_board.generate_xml()
    my_board.generate_define_device_io_header()
    
    filename=my_board.get_filename_str() + "/IO_data.pkl"
    with open(filename, 'wb') as output:
        pickle.dump(my_board, output, pickle.HIGHEST_PROTOCOL)
        
    print ("Config files generated.\n")
    raw_input(" Press ENTER to continue...")
        
    config_files_generated=True 
 

# Exit program
def exit():
    exit_program=True
    if (not config_files_generated):
        print ("XML file not generated.")
        print ("Exit?(Y/N).")   
        choice = raw_input(" >>  ")
        if (choice!='Y' and choice!='y'):
            exit_program=False
    
    if (exit_program):
        clear_screen()
        sys.exit()
   
# =======================
#    MENUS DEFINITIONS
# =======================

# Menu definition
menu_actions = {
    'main_menu': main_menu,
    '1': menu_board_info,
    '2': menu_iochange,
    '3': menu_generate_files,
    '0': exit,
}
 
 
# =======================
#      MAIN PROGRAM
# =======================
 
# Main Program
if __name__ == "__main__":
    global config_files_generated 

    #Check if argument were given and trie to open the .pkl file to load board object information
    if (len(sys.argv) > 1):
        folder = sys.argv[1]
        if(os.path.isdir(folder)):
            filename=folder + "/IO_data.pkl"
            with open(filename, 'rb') as input:
                my_board = pickle.load(input)
                config_files_generated=True
        else:
            print_error ("Unable to read directory.")
            sys.exit()
    else:
    #If no argument was given creaqte a new board
        my_board = IOBoard()
    
    # Launch main menu
    while True:
         menu_actions['main_menu']()
         
