/**
 * regtable
 *
 * Author: Alberto Pelarda
 * Creation date: 01/04/2016
 */

#include "product.h"
#include "regtable.h"

#include "define_device_io.h"
#include "define_type.h"

#include "errores.h"

/**
 * Definition of common registers
 */
DEFINE_COMMON_REGISTERS()


/**
 * setRegOutput
 *
 * Set value for a binary/PWM output
 *
 * 'rId'     Register ID
 * 'levels'  New PWM levels
 */
const void setOutput(byte idx, byte *state)
{

  // Launch error if register is not an output 
  if ( io_data[idx].reg_type!=BINARY_OUTPUT && io_data[idx].reg_type!=PWM_OUTPUT )
    error(ERROR_OUTPUT_NOT_AVAILABLE);
  else
  {
    // Update state in io_data
    io_data[idx].data[0]=state[0];
  
    // Set output
    if(io_data[idx].reg_type==BINARY_OUTPUT)
      digitalWrite(def_pins[idx], state[0]);
    else
      analogWrite(def_pins[idx], state[0]);
  }
}


/**
 * setRegOutput
 *
 * Set value for a binary/PWM output
 *
 * 'rId'     Register ID
 * 'levels'  New PWM levels
 */
const void setRegOutput(byte rId, byte *state)
{
  byte idx;
  
  // Get index of the register
  idx = rId - REG_IO_00;
  
  //Set output
  setOutput(idx,state);
  
  // Update register
  regTable[rId]->value[0] = state[0];

}


/**
 * Definition of custom registers
 */

/*******************************************************************************************************/

#if (IO_PORT00==BINARY_OUTPUT)
  REGISTER regBinIO_00(io_state_data_00, sizeof(io_state_data_00), NULL, &setRegOutput);
#else
  REGISTER regBinIO_00(io_state_data_00, sizeof(io_state_data_00), NULL, NULL);
#endif

#if (IO_PORT01==BINARY_OUTPUT)
  REGISTER regBinIO_01(io_state_data_01, sizeof(io_state_data_01), NULL, &setRegOutput);
#else
  REGISTER regBinIO_01(io_state_data_01, sizeof(io_state_data_01), NULL, NULL);
#endif

#if (IO_PORT02==BINARY_OUTPUT || IO_PORT02==PWM_OUTPUT)
  REGISTER regBinIO_02(io_state_data_02, sizeof(io_state_data_02), NULL, &setRegOutput);
#else
  REGISTER regBinIO_02(io_state_data_02, sizeof(io_state_data_02), NULL, NULL);
#endif

#if (IO_PORT03==BINARY_OUTPUT || IO_PORT03==PWM_OUTPUT)
  REGISTER regBinIO_03(io_state_data_03, sizeof(io_state_data_03), NULL, &setRegOutput);
#else
  REGISTER regBinIO_03(io_state_data_03, sizeof(io_state_data_03), NULL, NULL);
#endif

#if (IO_PORT04==BINARY_OUTPUT || IO_PORT04==PWM_OUTPUT)
  REGISTER regBinIO_04(io_state_data_04, sizeof(io_state_data_04), NULL, &setRegOutput);
#else
  REGISTER regBinIO_04(io_state_data_04, sizeof(io_state_data_04), NULL, NULL);
#endif

#if (IO_PORT05==BINARY_OUTPUT || IO_PORT05==PWM_OUTPUT)
  REGISTER regBinIO_05(io_state_data_05, sizeof(io_state_data_05), NULL, &setRegOutput);
#else
  REGISTER regBinIO_05(io_state_data_05, sizeof(io_state_data_05), NULL, NULL);
#endif

#if (IO_PORT06==BINARY_OUTPUT)
  REGISTER regBinIO_06(io_state_data_06, sizeof(io_state_data_06), NULL, &setRegOutput);
#else
  REGISTER regBinIO_06(io_state_data_06, sizeof(io_state_data_06), NULL, NULL);
#endif

#if (IO_PORT07==BINARY_OUTPUT)
  REGISTER regBinIO_07(io_state_data_07, sizeof(io_state_data_07), NULL, &setRegOutput);
#else
  REGISTER regBinIO_07(io_state_data_07, sizeof(io_state_data_07), NULL, NULL);
#endif

#if (IO_PORT08==BINARY_OUTPUT)
  REGISTER regBinIO_08(io_state_data_08, sizeof(io_state_data_08), NULL, &setRegOutput);
#else
  REGISTER regBinIO_08(io_state_data_08, sizeof(io_state_data_08), NULL, NULL);
#endif

#if (IO_PORT09==BINARY_OUTPUT)
  REGISTER regBinIO_09(io_state_data_09, sizeof(io_state_data_09), NULL, &setRegOutput);
#else
  REGISTER regBinIO_09(io_state_data_09, sizeof(io_state_data_09), NULL, NULL);
#endif

#if (IO_PORT10==BINARY_OUTPUT)
  REGISTER regBinIO_10(io_state_data_10, sizeof(io_state_data_10), NULL, &setRegOutput);
#else
  REGISTER regBinIO_10(io_state_data_10, sizeof(io_state_data_10), NULL, NULL);
#endif

#if (IO_PORT11==BINARY_OUTPUT)
  REGISTER regBinIO_11(io_state_data_11, sizeof(io_state_data_11), NULL, &setRegOutput);
#else
  REGISTER regBinIO_11(io_state_data_11, sizeof(io_state_data_11), NULL, NULL);
#endif

#if (IO_PORT11==BINARY_OUTPUT)
  REGISTER regBinIO_12(io_state_data_12, sizeof(io_state_data_12), NULL, &setRegOutput);
#else
  REGISTER regBinIO_12(io_state_data_12, sizeof(io_state_data_12), NULL, NULL);
#endif

#if (IO_PORT13==BINARY_OUTPUT)
  REGISTER regBinIO_13(io_state_data_13, sizeof(io_state_data_13), NULL, &setRegOutput);
#else
  REGISTER regBinIO_13(io_state_data_13, sizeof(io_state_data_13), NULL, NULL);
#endif

#if (IO_PORT14==BINARY_OUTPUT)
  REGISTER regBinIO_14(io_state_data_14, sizeof(io_state_data_14), NULL, &setRegOutput);
#else
  REGISTER regBinIO_14(io_state_data_14, sizeof(io_state_data_14), NULL, NULL);
#endif

#if (IO_PORT15==BINARY_OUTPUT)
  REGISTER regBinIO_15(io_state_data_15, sizeof(io_state_data_15), NULL, &setRegOutput);
#else
  REGISTER regBinIO_15(io_state_data_15, sizeof(io_state_data_15), NULL, NULL);
#endif

/*******************************************************************************************************/

/**
 * Initialize table of registers
 */
DECLARE_REGISTERS_START()
  &regBinIO_00,
  &regBinIO_01,
  &regBinIO_02,
  &regBinIO_03,
  &regBinIO_04,
  &regBinIO_05,
  &regBinIO_06,
  &regBinIO_07,
  &regBinIO_08,
  &regBinIO_09,
  &regBinIO_10,
  &regBinIO_11,
  &regBinIO_12,
  &regBinIO_13,
  &regBinIO_14,
  &regBinIO_15,
DECLARE_REGISTERS_END()

/**
 * Definition of common getter/setter callback functions
 */
DEFINE_COMMON_CALLBACKS()



