/**
 * io_v05
 *
 * Author: Alberto Pelarda
 * Creation date: 01/04/2016
 */
 
#include "swap.h"
#include "regtable.h"

#include "define_device_io.h"

#include "errores.h"


/**
 * Maximum repeating count
 */
const byte maxRepeaterHop = 5;


// RTC data
RTCDATA rtcData;

// Smooth read variables
#define NUM_VALUES_SMOOTH_READ 10
byte input_analog_read[NUM_IO_PORTS][NUM_VALUES_SMOOTH_READ];
int num_lect;



/**
 * SmoothAnalogInput_init
 *
 * Init values for smooth read for a analog input
 *
 */
void SmoothAnalogInput_init (void)
{
     unsigned int idx,i;
     num_lect=0;
     
     //Fill table with input analog values or 0
     for (i=0; i<NUM_VALUES_SMOOTH_READ; i++)
        for (idx=0; idx<NUM_IO_PORTS; idx++)
           {
               if(io_data[i].reg_type==ANALOG_INPUT)
	             {
                  uint16_t result;
                  uint16_t result_t;
                  byte result_f;
                  
                  
                  result=(analogRead(def_pins[idx]));
                  result_t= ((result/4095.0) * 255);
                  result_f=(byte)(constrain(result_t, 0, 255));
                  input_analog_read[idx][i] = result_f;
	             }
	             else
	                input_analog_read[idx][i]=0;
           }
     
     
}

/**
 * ReadSmoothAnalogInput
 *
 * Smooth read for a analog input
 *
 * 'rId'     Register ID
 */
byte ReadSmoothAnalogInput( int idx )
{
     unsigned int i, average;
     long sum;
     byte outputValue;
         
     // Read input analog value to the table
     //input_analog_read[idx][num_lect]=(byte)(analogRead(def_pins[idx]));
     if(io_data[idx].reg_type==ANALOG_INPUT)
     {
                  uint16_t result;
                  uint16_t result_t;
                  byte result_f;
                  
                  
                  result=(analogRead(def_pins[idx]));
                  result_t= ((result/4095.0) * 255);
                  result_f=(byte)(constrain(result_t, 0, 255));
                  input_analog_read[idx][num_lect] = result_f;
     }
     
     //advance to the next position in the array
     num_lect=(num_lect+1)%NUM_VALUES_SMOOTH_READ;

     //Calculate average value from the table
     sum=0;
     for (i=0; i<NUM_VALUES_SMOOTH_READ; i++)
        sum+=input_analog_read[idx][i];
     
     average=(int)sum/NUM_VALUES_SMOOTH_READ;
     
     // Re-maps the average to 0-255 range
     outputValue = map((int)average, 0, 255, 0, 9);
     //outputValue=average;
     
     return outputValue;
}

/**
 * readInput
 *
 * Read value for a binary input and return if a change was detected
 *
 * 'rId'     Register ID
 */
 
bool readInput( int idx)
{
     byte input_read;       // Input state information
     bool change_detected;  // Flag for input state change information
     
     change_detected=FALSE;
     
     // Launch error if register is not an input 
     if ( io_data[idx].reg_type!=BINARY_INPUT && io_data[idx].reg_type!=ANALOG_INPUT)
      error(ERROR_INPUT_NOT_AVAILABLE);
     else
     {
       // Read pin value
       if(io_data[idx].reg_type==BINARY_INPUT)
       {
        input_read=(byte)(digitalRead(def_pins[idx]));
       }
       else
       {
        /*uint16_t result;
        uint16_t result_t;
        uint16_t pwmresult;*/
        input_read=ReadSmoothAnalogInput(idx);
        /*result=(analogRead(def_pins[idx]));
        result_t= ((result/4095.0) * 255);
        pwmresult = (byte)(constrain(result_t, 0, 255));
        input_read = pwmresult; */
       }
         
       // If input state is diferent from io_data information set change_detected flag
       if ( io_data[idx].data[0]!=input_read)
       {  
        if (idx==12)
        {
          digitalWrite(LED, HIGH);
          delay(100);
          digitalWrite(LED, LOW);
        }
        change_detected=TRUE;
       }
       // Update value in io_data information
         io_data[idx].data[0] = input_read;
     }
     
     // return if a change in the input was detected 
     return change_detected;
}



/**
 * sendRegInput
 *
 * Send value for a binary input
 *
 * 'rId'     Register ID
 */
void sendRegInput (byte rId)
{
    // Send data
    swap.getRegister(rId)->getData();
}


/**
 * sendData
 *
 * Send value for a binary input
 *
 * 'rId'     Register ID
 */
void sendData (int idx)
{
    byte rId;
    
    // Get index of the register
    rId = idx + REG_IO_00;
    
    // Launch error if register is not an input 
    if ( io_data[idx].reg_type!=BINARY_INPUT && io_data[idx].reg_type!=ANALOG_INPUT)
      error(ERROR_INPUT_NOT_AVAILABLE);
    else
    {
      // Send data
      sendRegInput (rId);
    }
}


/**
 * configPorts
 *
 * Configure I/O ports
 */
void configPorts(void)
{
  unsigned int i;
  
  for (i=0;i<NUM_IO_PORTS;i++)
  {
     if( io_data[i].reg_type==BINARY_INPUT )
       pinMode(def_pins[i], INPUT);
     else if( io_data[i].reg_type==BINARY_OUTPUT || io_data[i].reg_type==PWM_OUTPUT )
     {
       pinMode(def_pins[i], OUTPUT);
       digitalWrite(def_pins[i], 0);
     }
     else if( io_data[i].reg_type==0 )
       pinMode(def_pins[i], INPUT_PULLUP);
  }
}



/**
 * setup
 *
 * Arduino setup function
 */
void setup()
{
  unsigned int i;

  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);

  // Configure ports
  configPorts();

  // Init values for analog inputs
  SmoothAnalogInput_init();

  // Init SWAP stack
  swap.init();

  // Enable repeater mode
  //swap.enableRepeater(maxRepeaterHop);
  
  // Transmit product code
  swap.getRegister(REGI_PRODUCTCODE)->getData();

  // Enter SYNC state
  swap.enterSystemState(SYSTATE_RXON);

  // During 3 seconds, listen the network for possible commands whilst the LED blinks
  for(i=0 ; i<6 ; i++)
  {
    digitalWrite(LED, HIGH);
    delay(100);
    digitalWrite(LED, LOW);
    delay(400);
  }

  

  // Transmit/Set initial binary states
  for(i=0 ; i<NUM_IO_PORTS ; i++)
  {
    if ( io_data[i].reg_type==BINARY_INPUT || io_data[i].reg_type==ANALOG_INPUT )
    {
      readInput(i);
      sendData(i);
    }
    else if ( io_data[i].reg_type==BINARY_OUTPUT)
    {
      setOutput(i,0);
    }
  }
}

/**
 * loop
 *
 * Arduino main loop
 */
void loop()
{
  
  bool data_change_detected[NUM_IO_PORTS];
  bool any_data_change;
  unsigned int i;

  //Read all inputs and check if any input has changed
  any_data_change=FALSE;
  for (i=0;i<NUM_IO_PORTS;i++)
  {
    data_change_detected[i]=FALSE;
    if ( io_data[i].reg_type==BINARY_INPUT || io_data[i].reg_type==ANALOG_INPUT )
    {
      data_change_detected[i]=readInput(i);
      any_data_change|=data_change_detected[i];
    }
  }
  
  // If any input has changed send new data
  if (any_data_change)
  {
    for (i=0;i<NUM_IO_PORTS;i++)
    {
      if ( ( io_data[i].reg_type==BINARY_INPUT || io_data[i].reg_type==ANALOG_INPUT ) && data_change_detected[i])
      {
        sendData(i);
      }
    }
  }
  delay(100);
}

